@@ -172,7 +172,7 @@ impl OpState {
       op_table: OpTable::default(),
       get_error_class_fn: &|_| "Error",
       tracker: OpsTracker {
-        ops: Vec::with_capacity(256),
+        ops: RefCell::new(Vec::with_capacity(256)),
       },
       gotham_state: Default::default(),
     }
